/*
 * psm_scan v0.2
 *
 * Copyright (C) 2003 Collin R. Mulliner <collin@betaversion.net>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY
 * RIGHTS.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE
 * FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS,
 * COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE
 * IS DISCLAIMED.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/poll.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>

#include <asm/types.h>
#include <asm/byteorder.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>

int psm_scan(char *dstAddr, char *srcAddr, unsigned short int s, unsigned short int e, int o)
{
	struct sockaddr_l2 sa;
	int sock;
	bdaddr_t src, dst;
	unsigned short int i;
	
	
	baswap(&dst, strtoba(dstAddr));
	baswap(&src, strtoba(srcAddr));

	for (i = s; i < e; i++) {
		sock = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);

		if (sock >= 0) {
			sa.l2_family = AF_BLUETOOTH;
			sa.l2_psm = 0;
		
			if (bacmp(&src, BDADDR_ANY) != 0) {
				sa.l2_bdaddr = src;
				if (bind(sock, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
					perror("Can't bind()");
				return(-1);
				}
			}
		
			sa.l2_psm = htobs(i);
			sa.l2_bdaddr = dst;

			if (connect(sock, (struct sockaddr *)&sa, sizeof(sa)) == 0)
				printf("0x%0.4x open\n",i);
			else {
				if (o == 1) {
					printf("0x%0.4x closed\n",i);
				}
			}
			
			close(sock);
		}
	}
	
	return(0);
}

void usage()
{
	printf("psm_scan v0.2\nusage:\n\tpsm_scan [-S <src bd_addr>] [-v] "\
	"[-s <psm>] [-e <psm>] <dst bd_addr>\n\tv\t: verbose output\n"\
	"\ts\t: start psm (0-0xffff), default 1\n\te\t: end psm (0-0xffff), default 0x1000\n");
}

int main(int argc, char **argv)
{
	char srcAddr[20];
	int opt;
	int oo = 0;
	int s = 1;
	int e = 0x1000;


	memcpy(srcAddr, "00:00:00:00:00:00\0", 18);
	while ((opt = getopt(argc, argv,"S:vs:e:")) != EOF) {
	
		switch(opt) {
		case 'S':
			strncpy(srcAddr, optarg, 20);
			break;
		case 's':
			s = atoi(optarg);
			if (s == 0) {
				usage();
				exit(1);
			}
			break;
		case 'e':
			e = atoi(optarg);
			if (e == 0) {
				usage();
				exit(1);
			}
			break;
		case 'v':
			oo = 1;
			break;
		default:
			usage();
			exit(1);
		}
	}

	if (!(argc - optind)) {
		usage();
		exit(1);
	}

	exit(psm_scan(argv[optind], srcAddr, s, e, oo));
}
